import ipaddress
import socket


def generate_ip_list(ip_range: str) -> list[str]:
    """
    Creates a list of IP addresses from provided CIDR range.
    :param ip_range: IP range in CIDR notation
    :return: list of IP addresses as strings
    """
    return [str(ip) for ip in ipaddress.IPv4Network(ip_range)]


def get_hostname_by_ip(ip: str) -> str:
    """
    Gets hostname by IP address.
    :param ip: IP to check as string.
    :return: Either the hostname as a string or 'Hostname not found'
    """
    try:
        return socket.gethostbyaddr(ip)[0]
    except:
        return 'Hostname not found'



import os
import inspect
import csv
from pprint import pprint


def retrieve_name(var) -> str:
    """
    Gets name of variable passed to a function.
    :param var: Variable whos name is needed.
    :return: String of original variable name.
    """
    callers_local_vars = inspect.currentframe().f_back.f_back.f_locals.items()
    return [var_name for var_name, var_val in callers_local_vars if var_val is var][0]


def write_result_csv(source: list[dict], method: str, prepend: str = None) -> None:
    """
    Writes a list of dictionaries to a CSV file in CWD. Name
    of file is generated from name of list variable. Requires
    'retrieve_name' function.
    :param source: List of dictionaries with same field names.
    :param method: Method of writing ('w', 'w+', 'a', 'a+').
    :param prepend: String to prepend to filename, optional.
    :return: None
    """
    if prepend is None:
        filename = f'{retrieve_name(source)}.csv'
    else:
        filename = f'{prepend}-{retrieve_name(source)}.csv'
    file_exists = os.path.exists(filename)
    fieldnames = list(source[0].keys())
    with open(filename, method) as f:
        writer = csv.DictWriter(f, fieldnames=fieldnames, lineterminator='\n', delimiter=',')
        if not file_exists:
            writer.writeheader()
        for data in source:
            writer.writerow(data)
            pprint(f"Writing Row: {data}")


def reformat_mac(mac: str) -> str:
    """
    Reformat MAC address to all lowercase with
     dashes separating octets e.g. 1a-2a-a3-b5-e4-2a
    :param mac: MAC address to format
    :return: reformatted MAC as string
    """
    mac = mac.replace('-', '').replace(':', '')
    mac = '-'.join(mac[i:i + 2] for i in range(0, 12, 2)).lower()
    return mac



import concurrent.futures
from typing import Callable
import time
from pprint import pprint


def multithread(function: Callable, switch_ips: list[str]) -> None:
    """
    Runs function concurrently on list of switch IP
    addresses. TODO: Fix messy output when running functions concurrently.
    :param function: Function to run on IPs
    :param switch_ips: List of switch IP addresses
    :return: None
    """
    with concurrent.futures.ThreadPoolExecutor() as executor:
        start = time.perf_counter()
        response_process = []
        for ip in switch_ips:
            response_process.append(executor.submit(function, ip))
        print(f'Duration: {time.perf_counter() - start}')
        for f in response_process[0:]:
            pprint(f.result())


if __name__ == '__main__':
    pass



import os

from netmiko import ConnectHandler
from halo import Halo


def ping_from_switch(coninfo: dict, ip_list: list[str]) -> None:
    """
    Pings list of IPs from switch. Used primarily for populating ARP table.
    :param coninfo: Dictionary of connection info
    :param ip_list: List of IP addresses to ping
    :return: None
    """
    spinner = Halo(spinner='dots')
    spinner.start(f'\nConnecting to {coninfo["ip"]}')
    connection = ConnectHandler(**coninfo)
    spinner.succeed()
    spinner.stop()
    for ip in ip_list:
        spinner.start(f'\nPinging {ip} from switch {coninfo["ip"]}')
        connection.send_command(f'ping {ip}')
        spinner.succeed()
        spinner.stop()
    connection.disconnect()


def run_commands(coninfo: dict, commands: list[str]) -> None:
    """
    Cycles through a list of commands to run on a switch
    :param coninfo: Dictionary with connecion info
    :param commands: Command to run on switch
    :return: None
    """
    spinner = Halo(spinner='dots')
    spinner.start(f'Connecting to {coninfo["ip"]}')
    connection = ConnectHandler(**coninfo)
    spinner.succeed()
    spinner.stop()
    for command in commands:
        spinner.start(f'\nRunning "{command}" on switch at {coninfo["ip"]}. This might take a bit.')
        return_data = connection.send_command(command)
        spinner.succeed()
        spinner.stop()
        command = command.replace(' ', '_').replace('-', '_')
        spinner.start(f'\nWriting {command} to switch_{command}/{coninfo["ip"]}')
        if not os.path.exists(f'switch_{command}'):
            os.mkdir(f'switch_{command}')
        with open(f'switch_{command}/{coninfo["ip"]}', 'w+') as f:
            f.write(return_data)
        spinner.succeed(f'\nCommand "show {command}" on {coninfo["ip"]} completed and written to switch_{command}/{coninfo["ip"]}')
        spinner.stop()
    spinner.start(f'\nClosing connection to {coninfo["ip"]}')
    connection.disconnect()
    spinner.succeed()
    spinner.stop()
